-- ContactAction
-- Author: grumblerbear
-- DateCreated: 08.09.2013 19:28:49
--------------------------------------------------------------
include( "FLuaVector" );
include( "UnitPanel" );

local redColor = Vector4( 0.7, 0, 0, 1 );
local highlightColor = Vector4( 0.7, 0.7, 0, 1 );

local NewCustomInterfaceModeChangeHandler = {};
local OldCustomInterfaceModeChangeHandler = {};

local defaultCursor = GameInfoTypes[GameInfo.InterfaceModes[InterfaceModeTypes.INTERFACEMODE_SELECTION].CursorType];

local InterfaceModeMessageHandler = {};
local DefaultMessageHandler = {};

local CurMission = 0;

-- iNumAirMission:
-- 1: BombBuilding
-- 2: BombImprovement
-- 3: PointedStrike
-- 4: Recon
function CanDoActionAt( iNumAirMission, pUnit, plotX, plotY )
	if (iNumAirMission == 1) then -- BombBuilding
		return pUnit:CanBombBuildingAt(plotX, plotY);
	end
	if (iNumAirMission == 2) then -- BombImprovement
		return pUnit:CanBombImprovementAt(plotX, plotY);
	end
	if (iNumAirMission == 3) then -- PointedStrike
		return pUnit:CanPointedStrikeAt(plotX, plotY);
	end
	if (iNumAirMission == 4) then -- Recon
		return pUnit:CanReconAt(plotX, plotY);
	end
	
	return false;
end

function DoActionAt( iNumAirMission, pUnit, plotX, plotY )
	if (CanDoActionAt( iNumAirMission, pUnit, plotX, plotY )) then
		print("DoActionAt" .. iNumAirMission);
		if (iNumAirMission == 1) then -- BombBuilding
			pUnit:AttackBombCity(plotX, plotY);
		end
		if (iNumAirMission == 2) then -- BombImprovement
			pUnit:AttackBombImprovement(plotX, plotY);
		end
		if (iNumAirMission == 3) then -- PointedStrike
			pUnit:AttackPointedStrike(plotX, plotY);
			--OnShowListPointedStrike(plotX, plotY);
		end
		if (iNumAirMission == 4) then -- Recon
			pUnit:AttackRecon(plotX, plotY);
		end
	else
		print("CanDoActionAt == false!!!");
	end
end

function ContactActionHighlight( iNumAirMission )
	if (iNumAirMission < 1 and iNumAirMission > 4) then
		return;
	end
	
	local pHeadSelectedUnit = UI.GetHeadSelectedUnit();
	local thingThatCanActuallyFire = nil;
	
	local iRange = pHeadSelectedUnit:Range();
	local bIndirectFireAllowed;
	local bDomainOnly;
	local thisPlot = nil;
	local thisX;
	local thisY;
	local thisTeam;

	thisPlot = pHeadSelectedUnit:GetPlot();
	thisX = pHeadSelectedUnit:GetX();
	thisY = pHeadSelectedUnit:GetY();
	thisTeam = pHeadSelectedUnit:GetTeam();
	thingThatCanActuallyFire = pHeadSelectedUnit;
	bDomainOnly = false;

	if thingThatCanActuallyFire ~= nil and thisPlot then
		-- highlight the bombardable plots
		local NO_DIRECTION = -1;
		for iDX = -iRange, iRange do
			for iDY = -iRange, iRange do
				--local pTargetPlot = Map.GetPlotXY(thisX, thisY, iDX, iDY);
				local pTargetPlot = Map.PlotXYWithRangeCheck(thisX, thisY, iDX, iDY, iRange);
				local bCanRangeStrike = true;

				if pTargetPlot then
					if not bIndirectFireAllowed then
						if not thisPlot:CanSeePlot(pTargetPlot, thisTeam, iRange - 1, NO_DIRECTION) then
							bCanRangeStrike = false;
						end
					end
					
					if bDomainOnly then
						if thingThatCanActuallyFire:GetDomainType() == DomainTypes.DOMAIN_LAND and pTargetPlot:IsWater() then
							bCanRangeStrike = false;
						elseif thingThatCanActuallyFire:GetDomainType() == DomainTypes.DOMAIN_SEA and not pTargetPlot:IsWater() then
							bCanRangeStrike = false;
						end
					end

					if bCanRangeStrike then
						if pTargetPlot:IsVisible(thisTeam, false) then
							local plotX = pTargetPlot:GetX();
							local plotY = pTargetPlot:GetY();
							local plotDistance = Map.PlotDistance(thisX, thisY, plotX, plotY);
							if plotDistance <= iRange then
								local hexID = ToHexFromGrid( Vector2( plotX, plotY) );
								
								if CanDoActionAt( iNumAirMission, pHeadSelectedUnit, plotX, plotY ) then
									if (iNumAirMission == 4) then -- Recon
										Events.SerialEventHexHighlight( hexID, true, highlightColor, "FireRangeBorder" );
									end
									if (iNumAirMission ~= 4) then -- Recon
										Events.SerialEventHexHighlight( hexID, true, redColor, "ValidFireTargetBorder");
									end
								else
									if (iNumAirMission == 4) then -- Recon
										Events.SerialEventHexHighlight( hexID, true, highlightColor, "FireRangeBorder" );
									end
								end
							end
						end
					end
				end
			end
		end
	end
end

function DisplayContactActionArrow( hexX, hexY )
	--find the selected attacker
	local unit = UI.GetHeadSelectedUnit();
	
	--get bombard end hex
	if CanDoActionAt( CurMission, unit, hexX, hexY ) then
		Events.SpawnArrowEvent( unit:GetX(), unit:GetY(), hexX, hexY );
	else
		Events.RemoveAllArrowsEvent();
	end
	
end

function BeginMission1Attack()
	local pHeadSelectedUnit = UI.GetHeadSelectedUnit();
	CurMission = 1;
	
	Events.SerialEventMouseOverHex.Add( DisplayContactActionArrow );
	ContactActionHighlight(1);
end
function BeginMission2Attack()
	local pHeadSelectedUnit = UI.GetHeadSelectedUnit();
	CurMission = 2;
	
	Events.SerialEventMouseOverHex.Add( DisplayContactActionArrow );
	ContactActionHighlight(2);
end
function BeginMission3Attack()
	local pHeadSelectedUnit = UI.GetHeadSelectedUnit();
	CurMission = 3;
	
	Events.SerialEventMouseOverHex.Add( DisplayContactActionArrow );
	ContactActionHighlight(3);
end
function BeginMission4Attack()
	local pHeadSelectedUnit = UI.GetHeadSelectedUnit();
	CurMission = 4;
	
	Events.SerialEventMouseOverHex.Add( DisplayContactActionArrow );
	ContactActionHighlight(4);
end

function EndContactActionAttack()
	CurMission = 0;
	Events.RemoveAllArrowsEvent();
	Events.SerialEventMouseOverHex.Remove( DisplayContactActionArrow );
	ClearUnitHexHighlights();
end

function ContactAction ( wParam, lParam )
	local plot = Map.GetPlot( UI.GetMouseOverHex() );
	local plotX = plot:GetX();
	local plotY = plot:GetY();
	LuaEvents.ContactAction ( plotX, plotY )
end

function ClearUnitHexHighlights()
	Events.ClearHexHighlightStyle(pathBorderStyle);
	Events.ClearHexHighlightStyle(attackPathBorderStyle);
	Events.ClearHexHighlightStyle(genericUnitHexBorder);  
	Events.ClearHexHighlightStyle("FireRangeBorder");
	Events.ClearHexHighlightStyle("GroupBorder");
	Events.ClearHexHighlightStyle("ValidFireTargetBorder");
end;

function OnInterfaceModeCustomChanged( oldInterfaceMode, newInterfaceMode)
	if (oldInterfaceMode ~= newInterfaceMode) then
		
		-- do any cleanup of the old mode
		local oldInterfaceModeHandler = OldCustomInterfaceModeChangeHandler[oldInterfaceMode];
		if oldInterfaceModeHandler then
			oldInterfaceModeHandler();
		end
		
		local cursorIndex;
		-- update the cursor to reflect this mode - these cursor are defined in Civ5CursorInfo.xml
		cursorIndex = GameInfoTypes[GameInfo.InterfaceModes[newInterfaceMode].CursorType];
		if cursorIndex then
			UIManager:SetUICursor(cursorIndex);
		else
			UIManager:SetUICursor(defaultCursor);
		end
		
		-- do any setup for the new mode
		local newInterfaceModeHandler = NewCustomInterfaceModeChangeHandler[newInterfaceMode];
		if newInterfaceModeHandler then
			newInterfaceModeHandler();
		end
	end
end
Events.InterfaceModeChanged.Add( OnInterfaceModeCustomChanged );
     
function InputCustomHandler( uiMsg, wParam, lParam )
	local interfaceMode = UI.GetInterfaceMode();
	local currentInterfaceModeHandler = InterfaceModeMessageHandler[interfaceMode];
	if currentInterfaceModeHandler and currentInterfaceModeHandler[uiMsg] then
		return currentInterfaceModeHandler[uiMsg]( wParam, lParam );
	elseif DefaultMessageHandler[uiMsg] then
		return DefaultMessageHandler[uiMsg]( wParam, lParam );
	end
	return false;
end
ContextPtr:SetInputHandler( InputCustomHandler );

InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_BOMBCITY] = {};
InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_IMPROVEMENT] = {};
InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_POINTEDSTRIKE] = {};
InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RECON] = {};

NewCustomInterfaceModeChangeHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_BOMBCITY] = BeginMission1Attack
OldCustomInterfaceModeChangeHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_BOMBCITY] = EndContactActionAttack
NewCustomInterfaceModeChangeHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_IMPROVEMENT] = BeginMission2Attack
OldCustomInterfaceModeChangeHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_IMPROVEMENT] = EndContactActionAttack
NewCustomInterfaceModeChangeHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_POINTEDSTRIKE] = BeginMission3Attack
OldCustomInterfaceModeChangeHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_POINTEDSTRIKE] = EndContactActionAttack
NewCustomInterfaceModeChangeHandler[GameInfoTypes.INTERFACEMODE_RECON] = BeginMission4Attack
OldCustomInterfaceModeChangeHandler[GameInfoTypes.INTERFACEMODE_RECON] = EndContactActionAttack

InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_BOMBCITY][MouseEvents.LButtonUp] = ContactAction;
InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_IMPROVEMENT][MouseEvents.LButtonUp] = ContactAction;
InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_POINTEDSTRIKE][MouseEvents.LButtonUp] = ContactAction;
InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RECON][MouseEvents.LButtonUp] = ContactAction;
if (UI.IsTouchScreenEnabled()) then
	InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_BOMBCITY][MouseEvents.PointerUp] = ContactAction;
	InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_IMPROVEMENT][MouseEvents.PointerUp] = ContactAction;
	InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_POINTEDSTRIKE][MouseEvents.PointerUp] = ContactAction;
	InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RECON][MouseEvents.PointerUp] = ContactAction;
else
	InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_BOMBCITY][MouseEvents.RButtonUp] = ContactAction;
	InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_IMPROVEMENT][MouseEvents.RButtonUp] = ContactAction;
	InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RANGE_ATTACK_POINTEDSTRIKE][MouseEvents.RButtonUp] = ContactAction;
	InterfaceModeMessageHandler[GameInfoTypes.INTERFACEMODE_RECON][MouseEvents.RButtonUp] = ContactAction;
end
