-------------------------------------------------
-- Diplomatic
-------------------------------------------------
include("IconSupport");
include("SupportFunctions.lua");
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler(bIsHide, bInitState)
	if (not bIsHide) then
		InitMajorCivList();
	end
end
ContextPtr:SetShowHideHandler(ShowHideHandler);


-------------------------------------------------
-------------------------------------------------
function InitMajorCivList()

	local g_iUs = Game.GetActivePlayer();
	local g_pUs = Players[ g_iUs ];
	local g_iUsTeam = g_pUs:GetTeam();
	local g_pUsTeam = Teams[ g_iUsTeam ];
	
	-- Clear buttons
	Controls.ItemStack:DestroyAllChildren();
	
	local count = 0;
	local iMajorMetCount = 0;
	--------------------------------------------------------------------
	-- Loop through all the Majors the active player knows
	--------------------------------------------------------------------
	for iOtherPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
		
		local pOtherPlayer = Players[ iOtherPlayer ];
		local iOtherTeam = pOtherPlayer:GetTeam();
		local pOtherTeam = Teams[ iOtherTeam ];
		
		-- Valid player? - Can't be us, and has to be alive
		if (iOtherPlayer ~= g_iUs and pOtherPlayer:IsAlive()) then

			-- Met this player?
			if (g_pUsTeam:IsHasMet(iOtherTeam)) then
			
				count = count + 1;
				iMajorMetCount = iMajorMetCount + 1;
				
				local controlTable = {};
				ContextPtr:BuildInstanceForControl("MajorInstance", controlTable, Controls.ItemStack);
				
				local primaryColor, secondaryColor = pOtherPlayer:GetPlayerColors();
				local civType = pOtherPlayer:GetCivilizationType();
				local civInfo = GameInfo.Civilizations[ civType ];
				
				local otherLeaderType = pOtherPlayer:GetLeaderType();
				local otherLeaderInfo = GameInfo.Leaders[otherLeaderType];
		
				IconHookup(otherLeaderInfo.PortraitIndex, 128, otherLeaderInfo.IconAtlas, controlTable.LeaderPortrait);
				CivIconHookup(iOtherPlayer, 64, controlTable.CivSymbol, controlTable.CivIconBG, controlTable.CivIconShadow, false, true);

				-- Player name
				local otherName;
				if (pOtherPlayer:IsHuman()) then
					otherName = pOtherPlayer:GetNickName();
				else
					otherName = pOtherPlayer:GetName();
				end
				TruncateString( controlTable.LeaderName, 140, otherName );
				
				
				-- Civ name
				controlTable.CivName:LocalizeAndSetText(civInfo.ShortDescription);
				
				-- Civ Era
				local iEra = pOtherPlayer:GetCurrentEra();
				controlTable.EraLabel:SetText(Locale.ConvertTextKey(GameInfo.Eras[iEra].Description));
				
				local iIdeology = pOtherPlayer:GetLateGamePolicyTree();
				if (iIdeology > -1) then
					controlTable.IdeologyLabel:SetText(Locale.ConvertTextKey(GameInfo.PolicyBranchTypes[iIdeology].Description));
					controlTable.IdeologyLabel:SetHide(false);
				else
					controlTable.IdeologyLabel:SetHide(true);
				end
				
				-- What happens if we click this?
				controlTable.Button:SetVoid1(iOtherPlayer);
				controlTable.Button:RegisterCallback(Mouse.eLClick, LeaderSelected);
				
				if (g_pUsTeam:HasEmbassyAtTeam(iOtherTeam) or g_pUs:IsMySpyVisitThem(iOtherPlayer) or g_pUs:IsMyDiplomatVisitingThem(iOtherPlayer)) then
					-- Resources
					for pResource in GameInfo.Resources() do
						local iResourceLoop = pResource.ID;
						
						iNumAvailable = pOtherPlayer:GetNumResourceAvailable(iResourceLoop, true);
						iNumMyself = pOtherPlayer:GetNumResourceAvailable(iResourceLoop, false);
						iNumTotal = pOtherPlayer:GetNumResourceTotal(iResourceLoop, true);
						iNumTotalMyself = pOtherPlayer:GetNumResourceTotal(iResourceLoop, false);
						iNumImport = pOtherPlayer:GetResourceImport(iResourceLoop);
						iNumFromMinors = pOtherPlayer:GetResourceFromMinors(iResourceLoop);
						iNumUsed = pOtherPlayer:GetNumResourceUsed(iResourceLoop);
						iNumExport = pOtherPlayer:GetResourceExport(iResourceLoop);
						iNumToMaster = pOtherPlayer:GetResourceExportToMaster(iResourceLoop);
						iNumFromVassals = pOtherPlayer:GetResourceImportFromVassals(iResourceLoop);
						
						if (iNumTotal > 0 or iNumUsed > 0 or iNumAvailable > 0) then
							if (Game.GetResourceUsageType(iResourceLoop) == ResourceUsageTypes.RESOURCEUSAGE_STRATEGIC or Game.GetResourceUsageType(iResourceLoop) == ResourceUsageTypes.RESOURCEUSAGE_LUXURY) then
								local textControlsResHeader = {};
								if (Game.GetResourceUsageType(iResourceLoop) == ResourceUsageTypes.RESOURCEUSAGE_STRATEGIC) then
									ContextPtr:BuildInstanceForControl("LTextEntry", textControlsResHeader, controlTable.StrategicStack);
								else
									ContextPtr:BuildInstanceForControl("LTextEntry", textControlsResHeader, controlTable.LuxuryStack);
								end
								
								local strTextResHeader;
								if (iNumAvailable >= 0) then
									strTextResHeader = Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO", iNumAvailable, pResource.IconString, Locale.ConvertTextKey(pResource.Description));
								else
									strTextResHeader = Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO2", iNumAvailable, pResource.IconString, Locale.ConvertTextKey(pResource.Description));
								end
								textControlsResHeader.Text:SetText("[ICON_BULLET]" .. strTextResHeader .. "[NEWLINE]");
								
								local strText = Locale.ConvertTextKey("TXT_KEY_RESOURCE_SOURCE");
								--local textControls = {};
								if ( iNumTotalMyself > 0) then
									strText = strText .. "[NEWLINE] [ICON_BULLET]" .. Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO_MYSELF", iNumTotalMyself);
								end
								if ( iNumImport > 0) then
									strText = strText .. "[NEWLINE] [ICON_BULLET]" .. Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO_IMPORT", iNumImport);
								end
								if ( iNumFromMinors > 0) then
									strText = strText .. "[NEWLINE] [ICON_BULLET]" .. Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO_MINOR", iNumFromMinors);
								end
								if ( iNumFromVassals > 0) then
									strText = strText .. "[NEWLINE] [ICON_BULLET]" .. Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO_FROMVASSALS", iNumFromVassals);
								end
								if ( iNumUsed > 0) then
									strText = strText .. "[NEWLINE] [ICON_BULLET]" .. Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO_USED", iNumUsed);
								end
								if ( iNumExport > 0) then
									strText = strText .. "[NEWLINE] [ICON_BULLET]" .. Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO_EXPORT", iNumExport);
								end
								if ( iNumToMaster > 0) then
									strText = strText .. "[NEWLINE] [ICON_BULLET]" .. Locale.ConvertTextKey("TXT_KEY_RESOURCE_STRINFO_TOMASTER", iNumToMaster);
								end
								textControlsResHeader.Text:SetToolTipString(strText);
							end
						end
					end
				else
					local textControls = {};
					ContextPtr:BuildInstanceForControl("LTextEntry", textControls, controlTable.StrategicStack);
					local strText = Locale.ConvertTextKey("TXT_KEY_DO_RESOURCE_DIPLO_PANEL_CANNTSEE");
					textControls.Text:SetText(strText);
				end
				
				controlTable.LuxuryStack:CalculateSize();
				controlTable.StrategicStack:CalculateSize();
				
				local sizeY = math.max(controlTable.LuxuryStack:GetSizeY(), controlTable.StrategicStack:GetSizeY());
				if (sizeY < 300) then 
					sizeY = 300;
				end				
				controlTable.Button:SetSizeY(sizeY + 5);
				controlTable.ButtonAnim:SetSizeY(sizeY);
				controlTable.ButtonAnimGrid:SetSizeY(sizeY + 5);
			end
		end
	end

	if (iMajorMetCount > 0) then
		Controls.NoMajorCivs:SetHide(true);
	else
		Controls.NoMajorCivs:SetHide(false);
	end
	
	Controls.ItemStack:CalculateSize();
	Controls.ItemStack:ReprocessAnchoring();
	Controls.MajorCivScrollPanel:CalculateInternalSize();
end



-------------------------------------------------
-- On Leader Selected
-------------------------------------------------
function LeaderSelected(ePlayer)

	if not Players[Game.GetActivePlayer()]:IsTurnActive() or Game.IsProcessingMessages() then
		return;
	end
	
	if (Players[ ePlayer ]:IsHuman()) then
		Events.OpenPlayerDealScreenEvent(ePlayer);
	else
		
		UI.SetRepeatActionPlayer(ePlayer);
		UI.ChangeStartDiploRepeatCount(1);
		Players[ ePlayer ]:DoBeginDiploWithHuman();	

	end
end
