-------------------------------------------------
-- Diplomatic
-------------------------------------------------
include("IconSupport");
include("SupportFunctions.lua");
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
function ShowHideHandler(bIsHide, bInitState)
	if (not bIsHide) then
		InitMajorCivList();
	end
end
ContextPtr:SetShowHideHandler(ShowHideHandler);


-------------------------------------------------
-------------------------------------------------
function InitMajorCivList()

	local g_iUs = Game.GetActivePlayer();
	local g_pUs = Players[ g_iUs ];
	local g_iUsTeam = g_pUs:GetTeam();
	local g_pUsTeam = Teams[ g_iUsTeam ];
	
	-- Clear buttons
	Controls.ItemStack:DestroyAllChildren();
	
	local count = 0;
	local iMajorMetCount = 0;
	--------------------------------------------------------------------
	-- Loop through all the Majors the active player knows
	--------------------------------------------------------------------
	for iOtherPlayer = 0, GameDefines.MAX_MAJOR_CIVS - 1 do
		
		local pOtherPlayer = Players[ iOtherPlayer ];
		local iOtherTeam = pOtherPlayer:GetTeam();
		local pOtherTeam = Teams[ iOtherTeam ];
		
		-- Valid player? - Can't be us, and has to be alive
		if (iOtherPlayer ~= g_iUs and pOtherPlayer:IsAlive()) then

			-- Met this player?
			if (g_pUsTeam:IsHasMet(iOtherTeam)) then
			
				count = count + 1;
				iMajorMetCount = iMajorMetCount + 1;
				
				local controlTable = {};
				ContextPtr:BuildInstanceForControl("MajorInstance", controlTable, Controls.ItemStack);
				
				local primaryColor, secondaryColor = pOtherPlayer:GetPlayerColors();
				local civType = pOtherPlayer:GetCivilizationType();
				local civInfo = GameInfo.Civilizations[ civType ];
				
				local otherLeaderType = pOtherPlayer:GetLeaderType();
				local otherLeaderInfo = GameInfo.Leaders[otherLeaderType];
		
				IconHookup(otherLeaderInfo.PortraitIndex, 128, otherLeaderInfo.IconAtlas, controlTable.LeaderPortrait);
				CivIconHookup(iOtherPlayer, 64, controlTable.CivSymbol, controlTable.CivIconBG, controlTable.CivIconShadow, false, true);

				-- Player name
				local otherName;
				if (pOtherPlayer:IsHuman()) then
					otherName = pOtherPlayer:GetNickName();
				else
					otherName = pOtherPlayer:GetName();
				end
				TruncateString( controlTable.LeaderName, 140, otherName );
				
				
				-- Civ name
				controlTable.CivName:LocalizeAndSetText(civInfo.ShortDescription);
				
				-- Civ Era
				local iEra = pOtherPlayer:GetCurrentEra();
				controlTable.EraLabel:SetText(Locale.ConvertTextKey(GameInfo.Eras[iEra].Description));
				
				-- REALISM MOD START
				local iIdeology = pOtherPlayer:GetLateGamePolicyTree();
				if (iIdeology > -1) then
					controlTable.IdeologyLabel:SetText(Locale.ConvertTextKey(GameInfo.PolicyBranchTypes[iIdeology].Description));
					controlTable.IdeologyLabel:SetHide(false);
				else
					controlTable.IdeologyLabel:SetHide(true);
				end
				-- REALISM MOD END
				
				-- What happens if we click this?
				controlTable.Button:SetVoid1(iOtherPlayer);
				controlTable.Button:RegisterCallback(Mouse.eLClick, LeaderSelected);
				
				if (g_pUsTeam:HasEmbassyAtTeam(iOtherTeam) or g_pUs:IsMySpyVisitThem(iOtherPlayer) or g_pUs:IsMyDiplomatVisitingThem(iOtherPlayer)) then
					-- Techs
					for pTech in GameInfo.Technologies() do
						local iTech = pTech.ID;
						
						if (pOtherTeam:GetTeamTechs():HasTech(iTech)) then
							if (not g_pUsTeam:GetTeamTechs():HasTech(iTech)) then
								local textControls = {};
								ContextPtr:BuildInstanceForControl("LTextEntry", textControls, controlTable.TechsStack);
								local strText = Locale.ConvertTextKey(pTech.Description);
								textControls.Text:SetText(strText);
							end
						end
						if (not pOtherTeam:GetTeamTechs():HasTech(iTech) and g_pUsTeam:GetTeamTechs():HasTech(iTech)) then
							local textControls = {};
							ContextPtr:BuildInstanceForControl("LTextEntry", textControls, controlTable.HasNotTechsStack);
							local strText = Locale.ConvertTextKey(pTech.Description);
							textControls.Text:SetText(strText);
						end
					end
				else
					local textControls = {};
					ContextPtr:BuildInstanceForControl("LTextEntry", textControls, controlTable.TechsStack);
					local strText = Locale.ConvertTextKey("TXT_KEY_DO_TECH_DIPLO_PANEL_CANNTSEE");
					textControls.Text:SetText(strText);
				end
				
				controlTable.TechsStack:CalculateSize();
				controlTable.HasNotTechsStack:CalculateSize();
				local sizeY = math.max(controlTable.TechsStack:GetSizeY(), controlTable.HasNotTechsStack:GetSizeY());
				if (sizeY < 300) then 
					sizeY = 300;
				end				
				controlTable.Button:SetSizeY(sizeY + 5);
				controlTable.ButtonAnim:SetSizeY(sizeY);
				controlTable.ButtonAnimGrid:SetSizeY(sizeY + 5);
			end
		end
	end

	if (iMajorMetCount > 0) then
		Controls.NoMajorCivs:SetHide(true);
	else
		Controls.NoMajorCivs:SetHide(false);
	end
	
	Controls.ItemStack:CalculateSize();
	Controls.ItemStack:ReprocessAnchoring();
	Controls.MajorCivScrollPanel:CalculateInternalSize();
end



-------------------------------------------------
-- On Leader Selected
-------------------------------------------------
function LeaderSelected(ePlayer)

	if not Players[Game.GetActivePlayer()]:IsTurnActive() or Game.IsProcessingMessages() then
		return;
	end
	
	if (Players[ ePlayer ]:IsHuman()) then
		Events.OpenPlayerDealScreenEvent(ePlayer);
	else
		
		UI.SetRepeatActionPlayer(ePlayer);
		UI.ChangeStartDiploRepeatCount(1);
		Players[ ePlayer ]:DoBeginDiploWithHuman();	

	end
end
